# TCL File Generated by Component Editor 15.0
# Sat May 23 16:39:31 EDT 2015
# DO NOT MODIFY


#
# command_uart "BladeRF Command UART" v1.0
# Nuand, LLC 2015.05.23.16:39:31
# bladeRF Command UART
#

#
# request TCL package from ACDS 15.0
#
package require -exact qsys 15.0


#
# module command_uart
#
set_module_property DESCRIPTION "bladeRF Command UART"
set_module_property NAME command_uart
set_module_property VERSION 1.0
set_module_property INTERNAL false
set_module_property OPAQUE_ADDRESS_MAP true
set_module_property GROUP "Interface Protocols/Serial"
set_module_property AUTHOR "Nuand, LLC"
set_module_property DISPLAY_NAME "BladeRF Command UART"
set_module_property INSTANTIATE_IN_SYSTEM_MODULE true
set_module_property EDITABLE true
set_module_property REPORT_TO_TALKBACK false
set_module_property ALLOW_GREYBOX_GENERATION false
set_module_property REPORT_HIERARCHY false


#
# file sets
#
add_fileset QUARTUS_SYNTH QUARTUS_SYNTH "" ""
set_fileset_property QUARTUS_SYNTH TOP_LEVEL command_uart
set_fileset_property QUARTUS_SYNTH ENABLE_RELATIVE_INCLUDE_PATHS false
set_fileset_property QUARTUS_SYNTH ENABLE_FILE_OVERWRITE_MODE false
add_fileset_file ../vhdl/command_uart.vhd VHDL PATH ../vhdl/command_uart.vhd TOP_LEVEL_FILE


#
# parameters
#


#
# display items
#


#
# connection point reset
#
add_interface reset reset end
set_interface_property reset associatedClock clock
set_interface_property reset synchronousEdges DEASSERT
set_interface_property reset ENABLED true
set_interface_property reset EXPORT_OF ""
set_interface_property reset PORT_NAME_MAP ""
set_interface_property reset CMSIS_SVD_VARIABLES ""
set_interface_property reset SVD_ADDRESS_GROUP ""

add_interface_port reset reset reset Input 1


#
# connection point rs232
#
add_interface rs232 conduit end
set_interface_property rs232 associatedClock ""
set_interface_property rs232 associatedReset ""
set_interface_property rs232 ENABLED true
set_interface_property rs232 EXPORT_OF ""
set_interface_property rs232 PORT_NAME_MAP ""
set_interface_property rs232 CMSIS_SVD_VARIABLES ""
set_interface_property rs232 SVD_ADDRESS_GROUP ""

add_interface_port rs232 rs232_sin serial_in Input 1
add_interface_port rs232 rs232_sout serial_out Output 1


#
# connection point interrupt
#
add_interface interrupt interrupt end
set_interface_property interrupt associatedAddressablePoint avalon_slave
set_interface_property interrupt bridgedReceiverOffset ""
set_interface_property interrupt bridgesToReceiver ""
set_interface_property interrupt ENABLED true
set_interface_property interrupt EXPORT_OF ""
set_interface_property interrupt PORT_NAME_MAP ""
set_interface_property interrupt CMSIS_SVD_VARIABLES ""
set_interface_property interrupt SVD_ADDRESS_GROUP ""

add_interface_port interrupt irq irq Output 1


#
# connection point clock
#
add_interface clock clock end
set_interface_property clock clockRate 0
set_interface_property clock ENABLED true
set_interface_property clock EXPORT_OF ""
set_interface_property clock PORT_NAME_MAP ""
set_interface_property clock CMSIS_SVD_VARIABLES ""
set_interface_property clock SVD_ADDRESS_GROUP ""

add_interface_port clock clock clk Input 1


#
# connection point avalon_slave
#
add_interface avalon_slave avalon end
set_interface_property avalon_slave addressUnits SYMBOLS
set_interface_property avalon_slave associatedClock clock
set_interface_property avalon_slave associatedReset reset
set_interface_property avalon_slave bitsPerSymbol 8
set_interface_property avalon_slave burstOnBurstBoundariesOnly false
set_interface_property avalon_slave burstcountUnits SYMBOLS
set_interface_property avalon_slave explicitAddressSpan 32
set_interface_property avalon_slave holdTime 0
set_interface_property avalon_slave linewrapBursts false
set_interface_property avalon_slave maximumPendingReadTransactions 1
set_interface_property avalon_slave maximumPendingWriteTransactions 0
set_interface_property avalon_slave readLatency 0
set_interface_property avalon_slave readWaitTime 1
set_interface_property avalon_slave setupTime 0
set_interface_property avalon_slave timingUnits Cycles
set_interface_property avalon_slave writeWaitTime 0
set_interface_property avalon_slave ENABLED true
set_interface_property avalon_slave EXPORT_OF ""
set_interface_property avalon_slave PORT_NAME_MAP ""
set_interface_property avalon_slave CMSIS_SVD_VARIABLES ""
set_interface_property avalon_slave SVD_ADDRESS_GROUP ""

add_interface_port avalon_slave addr address Input 5
add_interface_port avalon_slave dout readdata Output 32
add_interface_port avalon_slave din writedata Input 32
add_interface_port avalon_slave read read Input 1
add_interface_port avalon_slave write write Input 1
add_interface_port avalon_slave readack readdatavalid Output 1
add_interface_port avalon_slave waitreq waitrequest Output 1
set_interface_assignment avalon_slave embeddedsw.configuration.isFlash 0
set_interface_assignment avalon_slave embeddedsw.configuration.isMemoryDevice 0
set_interface_assignment avalon_slave embeddedsw.configuration.isNonVolatileStorage 0
set_interface_assignment avalon_slave embeddedsw.configuration.isPrintableDevice 0
